/*
 * $Id: ChainedTilesContextFactoryTest.java 674916 2008-07-08 19:06:05Z apetrelli $
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.tiles.context;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.apache.tiles.TilesApplicationContext;
import org.apache.tiles.mock.RepeaterTilesContextFactory;
import org.easymock.EasyMock;

/**
 * @version $Rev: 674916 $ $Date: 2008-07-08 21:06:05 +0200 (Tue, 08 Jul 2008) $
 */
public class ChainedTilesContextFactoryTest extends TestCase {

    /**
     * The Tiles application context.
     */
    private TilesApplicationContext appContext;

    /**
     * The request context.
     */
    private TilesRequestContext requestContext;

    /** {@inheritDoc} */
    @Override
    protected void setUp() throws Exception {
        super.setUp();
        appContext = EasyMock.createMock(TilesApplicationContext.class);
        requestContext = EasyMock.createMock(TilesRequestContext.class);
        EasyMock.replay(appContext);
    }

    /**
     * Tests the initialization method.
     *
     * @throws Exception If something goes wrong during testing.
     */
    public void testInit() throws Exception {
        Map<String, String> config = new HashMap<String, String>();
        config.put(ChainedTilesContextFactory.FACTORY_CLASS_NAMES,
                "this.is.not.a.class.Name,"
                + RepeaterTilesContextFactory.class.getName());
        ChainedTilesContextFactory factory = new ChainedTilesContextFactory();
        factory.init(config);
        TilesRequestContext context = factory.createRequestContext(appContext, requestContext);
        assertNotNull("The request context cannot be null", context);
    }
}
